from pydantic import Field

from tiny_brave.datamodels.requests.base import BaseSearchRequest


class VideoSearchRequest(BaseSearchRequest):
    ui_lang: str | None = Field(
        default='en-US',
        description=(
            'User interface language preferred in response. '
            'Format <language_code>-<country_code>.'
        ),
    )

    count: int = Field(
        default=20,
        ge=1,
        le=50,
        description=('The number of search results returned. Maximum is 50.'),
    )

    offset: int = Field(
        default=0,
        ge=0,
        le=9,
        description=('Zero-based offset for pagination. Maximum is 9.'),
    )

    safesearch: str | None = Field(
        default='moderate', description=('Filter adult content: off, moderate, strict.')
    )

    freshness: str | None = Field(
        default=None,
        description=(
            'Filter results by discovery time. '
            'Supported: pd, pw, pm, py, or YYYY-MM-DDtoYYYY-MM-DD.'
        ),
    )

    operators: bool = Field(
        default=True, description='Whether to apply search operators.'
    )



from typing import Literal

from pydantic import Field

from tiny_brave.datamodels.requests.base import BaseSearchRequest


class WebSearchRequest(BaseSearchRequest):
    country: str | None = Field(
        default='US',
        description='2-character country code for where the results come from.',
    )

    ui_lang: str | None = Field(
        default='en-US', description='UI language preferred in response (e.g. en-US).'
    )

    count: int = Field(
        default=3,
        ge=1,
        le=20,
        description='Number of search results to return (max 20).',
    )

    offset: int = Field(
        default=0,
        ge=0,
        le=9,
        description='Zero-based page offset for pagination (max 9).',
    )

    safesearch: Literal['off', 'moderate', 'strict'] = Field(
        default='moderate', description='Adult content filter level.'
    )

    freshness: Literal['pd', 'pw', 'pm', 'py'] | None = Field(
        default=None,
        description=(
            'Time filter for results. '
            'Options: pd (24h), pw (7d), pm (31d), py (365d), '
            'or custom YYYY-MM-DDtoYYYY-MM-DD'
        ),
    )

    text_decorations: bool = Field(
        default=True, description='Whether snippets include decoration markers.'
    )

    result_filter: str | None = Field(
        default=None,
        description=(
            "Comma-delimited result types to include (e.g. 'discussions,videos,web')."
        ),
    )

    goggles_id: str | None = Field(
        default=None, description='Deprecated: use `goggles`. Goggles ID string.'
    )

    goggles: list[str] | None = Field(
        default=None, description='List of goggle URLs or definitions for re-ranking.'
    )

    units: Literal['metric', 'imperial'] | None = Field(
        default=None, description='Measurement system. Default inferred from country.'
    )

    extra_snippets: bool | None = Field(
        default=None,
        description='If true, return up to 5 additional alternative snippets.',
    )

    summary: bool | None = Field(
        default=None, description='If true, enables summary key generation in results.'
    )

    operators: bool = Field(
        default=True, description='Whether to apply search operators.'
    )



from typing import List

from pydantic import Field

from tinygent.core.types.base import TinyModel


class Query(TinyModel):
    original: str = Field(..., description='The original query that was requested.')
    altered: str | None = Field(
        default=None, description='The altered query by the spellchecker (if any).'
    )
    cleaned: str | None = Field(
        default=None,
        description='The cleaned normalized query by the spellchecker (if any).',
    )
    spellcheck_off: bool | None = Field(
        default=None, description='Whether the spellchecker is enabled or disabled.'
    )
    show_strict_warning: bool | None = Field(
        default=None,
        description=(
            'True if results are missing due to strict safesearch. '
            'Adult content was blocked by safesearch.'
        ),
    )


class Thumbnail(TinyModel):
    src: str = Field(..., description='The served URL of the thumbnail.')
    original: str | None = Field(
        default=None, description='The original URL of the thumbnail.'
    )


class Profile(TinyModel):
    name: str = Field(..., description='The name of the profile.')
    long_name: str | None = Field(
        default=None, description='The long name of the profile.'
    )
    url: str = Field(..., description='The original URL where the profile is available.')
    img: str | None = Field(
        default=None, description='The served image URL representing the profile.'
    )


class VideoData(TinyModel):
    duration: str | None = Field(
        default=None,
        description='A time string representing the duration of the video.',
    )
    views: int | None = Field(
        default=None, description='The number of views of the video.'
    )
    creator: str | None = Field(default=None, description='The creator of the video.')
    publisher: str | None = Field(
        default=None, description='The publisher of the video.'
    )
    requires_subscription: bool | None = Field(
        default=None, description='Whether the video requires a subscription.'
    )
    tags: List[str] | None = Field(
        default=None, description='A list of tags relevant to the video.'
    )
    author: Profile | None = Field(
        default=None, description='A profile associated with the video.'
    )


class MetaUrl(TinyModel):
    scheme: str | None = Field(
        default=None, description='The protocol scheme extracted from the URL.'
    )
    netloc: str | None = Field(
        default=None, description='The network location part extracted from the URL.'
    )
    hostname: str | None = Field(
        default=None, description='The lowercased domain name extracted from the URL.'
    )
    favicon: str | None = Field(
        default=None, description='The favicon used for the URL.'
    )
    path: str | None = Field(
        default=None,
        description='The hierarchical path of the URL useful as a display string.',
    )


class VideoResult(TinyModel):
    type: str = Field(
        'video_result',
        description='The type of video search API result. Always video_result.',
    )
    url: str = Field(..., description='The source URL of the video.')
    title: str = Field(..., description='The title of the video.')
    description: str | None = Field(
        default=None, description='The description for the video.'
    )
    age: str | None = Field(
        default=None, description='A human readable representation of the page age.'
    )
    page_age: str | None = Field(
        default=None, description='The page age found from the source web page.'
    )
    page_fetched: str | None = Field(
        default=None,
        description='The ISO date time when the page was last fetched (YYYY-MM-DDTHH:MM:SSZ).',
    )
    thumbnail: Thumbnail | None = Field(
        default=None, description='The thumbnail for the video.'
    )
    video: VideoData | None = Field(default=None, description='Metadata for the video.')
    meta_url: MetaUrl | None = Field(
        default=None,
        description='Aggregated information on the URL associated with the video result.',
    )


class Extra(TinyModel):
    might_be_offensive: bool = Field(
        ...,
        description='Indicates whether the video search results might contain offensive content.',
    )


class VideoSearchApiResponse(TinyModel):
    type: str = Field(
        'videos', description='The type of search API result. Always videos.'
    )
    query: Query = Field(..., description='Video search query string.')
    results: List[VideoResult] = Field(
        ..., description='The list of video results for the given query.'
    )
    extra: Extra = Field(
        ..., description='Additional information about the video search results.'
    )



import logging
import os
from typing import TypeVar
from urllib.parse import urljoin

import httpx

from tiny_brave.constants import BASE_URL
from tiny_brave.constants import DEFAULT_MAX_RETRIES
from tiny_brave.constants import DEFAULT_TIMEOUT
from tiny_brave.datamodels.endpoints import BraveEndpoint
from tiny_brave.datamodels.requests.images import ImagesSearchReuest
from tiny_brave.datamodels.requests.news import NewsSearchRequest
from tiny_brave.datamodels.requests.videos import VideoSearchRequest
from tiny_brave.datamodels.requests.web import WebSearchRequest
from tiny_brave.datamodels.responses.images import ImageSearchApiResponse
from tiny_brave.datamodels.responses.news import NewsSearchApiResponse
from tiny_brave.datamodels.responses.videos import VideoSearchApiResponse
from tiny_brave.datamodels.responses.web import WebSearchApiResponse
from tiny_brave.exceptions import TinyBraveAPIError
from tiny_brave.exceptions import TinyBraveClientError
from tinygent.core.types.base import TinyModel

logger = logging.getLogger(__name__)

T = TypeVar('T', bound=TinyModel)

RETRYABLE_EXCEPTIONS = (
    httpx.ConnectError,
    httpx.ReadTimeout,
    httpx.RemoteProtocolError,
    httpx.WriteError,
)


class TinyBraveClient:
    def __init__(self):
        if not (brave_token := os.getenv('BRAVE_API_KEY')):
            raise TinyBraveClientError("'BRAVE_API_KEY' environment variable not set.")

        self._base_url = BASE_URL

        self._headers = {
            'X-Subscription-Token': brave_token,
            'Accept': 'application/json',
            'Accept-Encoding': 'gzip',
            'Cache-Control': 'no-cache',
        }

    async def _get(
        self,
        endpoint: BraveEndpoint,
        params: dict[str, str] | None = None,
        max_retries: int = DEFAULT_MAX_RETRIES,
        timeout: int = DEFAULT_TIMEOUT,
    ) -> httpx.Response:
        last_exc: Exception | None = None
        url = urljoin(self._base_url, f'{endpoint.value}/search')

        async with httpx.AsyncClient(
            headers=self._headers,
            timeout=timeout,
        ) as client:
            for attempt in range(1, max_retries + 1):
                try:
                    response = await client.get(url, params=params)
                    response.raise_for_status()
                    return response

                except RETRYABLE_EXCEPTIONS as e:
                    last_exc = e
                    logger.warning(
                        'Retryable network error calling %s: %s (attempt %d/%d)',
                        url,
                        e,
                        attempt,
                        max_retries,
                    )

                except httpx.HTTPStatusError as e:
                    logger.error(
                        'HTTP error calling %s: %s (status %d) - not retrying',
                        url,
                        e,
                        e.response.status_code,
                    )
                    raise TinyBraveClientError(
                        f'HTTP error {e.response.status_code} calling {url}: {e}'
                    )

            raise TinyBraveAPIError(
                f'Failed to fetch data from {url} after {max_retries} attempts.'
            ) from last_exc

    async def _use_brave(
        self,
        endpoint: BraveEndpoint,
        request: TinyModel,
        response_model: type[T],
        max_retries: int = DEFAULT_MAX_RETRIES,
        timeout: int = DEFAULT_TIMEOUT,
    ) -> T:
        response = await self._get(
            endpoint,
            params=request.model_dump(exclude_none=True, by_alias=True),
            max_retries=max_retries,
            timeout=timeout,
        )
        return response_model.model_validate(response.json())

    async def news(
        self,
        request: NewsSearchRequest,
        max_retries: int = DEFAULT_MAX_RETRIES,
        timeout: int = DEFAULT_TIMEOUT,
    ) -> NewsSearchApiResponse:
        return await self._use_brave(
            BraveEndpoint.news,
            request=request,
            response_model=NewsSearchApiResponse,
            max_retries=max_retries,
            timeout=timeout,
        )

    async def web(
        self,
        request: WebSearchRequest,
        max_retries: int = DEFAULT_MAX_RETRIES,
        timeout: int = DEFAULT_TIMEOUT,
    ) -> WebSearchApiResponse:
        return await self._use_brave(
            BraveEndpoint.web,
            request=request,
            response_model=WebSearchApiResponse,
            max_retries=max_retries,
            timeout=timeout,
        )

    async def images(
        self,
        request: ImagesSearchReuest,
        max_retries: int = DEFAULT_MAX_RETRIES,
        timeout: int = DEFAULT_TIMEOUT,
    ) -> ImageSearchApiResponse:
        return await self._use_brave(
            BraveEndpoint.images,
            request=request,
            response_model=ImageSearchApiResponse,
            max_retries=max_retries,
            timeout=timeout,
        )

    async def videos(
        self,
        request: VideoSearchRequest,
        max_retries: int = DEFAULT_MAX_RETRIES,
        timeout: int = DEFAULT_TIMEOUT,
    ) -> VideoSearchApiResponse:
        return await self._use_brave(
            BraveEndpoint.videos,
            request=request,
            response_model=VideoSearchApiResponse,
            max_retries=max_retries,
            timeout=timeout,
        )



from typing import Literal

from pydantic import Field

from tiny_brave.datamodels.requests.base import BaseSearchRequest


class ImagesSearchReuest(BaseSearchRequest):
    count: int = Field(
        default=3,
        ge=1,
        le=200,
        description='The number of search results to return (1–200).',
    )

    safesearch: Literal['off', 'strict'] | None = Field(
        default=None,
        description=(
            'Adult content filter level. Options are "off" or "strict". '
            'off - No filtering is done. '
            'strict - Drops all adult content from search results.'
        ),
    )



import re
from typing import Literal

from pydantic import Field
from pydantic import field_validator

from tiny_brave.datamodels.requests.base import BaseSearchRequest
from tiny_brave.exceptions import TinyBraveClientError


class NewsSearchRequest(BaseSearchRequest):
    ui_lang: str | None = Field(
        default='en-US', description='UI language preferred in response (e.g. en-US).'
    )

    count: int = Field(
        default=3,
        ge=1,
        le=50,
        description='The number of search results to return (1–50).',
    )

    offset: int = Field(
        default=0, le=9, description='Zero-based page offset for pagination (max 9).'
    )

    safesearch: Literal['off', 'moderate', 'strict'] = Field(
        default='moderate', description='Adult content filter level.'
    )

    freshness: str | None = Field(
        default=None,
        description=(
            'Time filter for results. '
            'Options: pd (24h), pw (7d), pm (31d), py (365d), '
            'or custom YYYY-MM-DDtoYYYY-MM-DD'
        ),
    )

    extra_snippets: bool | None = Field(
        default=None,
        description='If true, return up to 5 additional alternative snippets.',
    )

    goggles: list[str] | None = Field(
        default=None, description='List of goggle URLs or definitions for re-ranking.'
    )

    operators: bool = Field(
        default=True, description='Whether to apply search operators.'
    )

    @field_validator('country')
    def validate_country(cls, v: str | None) -> str | None:
        if v is not None and len(v) != 2:
            raise TinyBraveClientError('Country code must be exactly 2 characters.')
        return v

    @field_validator('ui_lang')
    def validate_ui_lang(cls, v: str | None) -> str | None:
        if v is None:
            return v
        # must be "xx" or "xx-XX"
        if not re.match(r'^[a-z]{2}(-[A-Z]{2})?$', v):
            raise TinyBraveClientError(
                f'Invalid ui_lang "{v}". Must be like "en" or "en-US".'
            )
        return v

    @field_validator('freshness')
    def validate_freshness(cls, v: str | None) -> str | None:
        if v is None:
            return v
        if v in ('pd', 'pw', 'pm', 'py'):
            return v
        if re.match(r'^\d{4}-\d{2}-\d{2}to\d{4}-\d{2}-\d{2}$', v):
            return v
        raise TinyBraveClientError(
            f'Invalid freshness "{v}". Must be one of pd, pw, pm, py, '
            f'or date range YYYY-MM-DDtoYYYY-MM-DD.'
        )



from typing import Any

from tiny_brave.client import TinyBraveClient
from tiny_brave.datamodels.requests.images import ImagesSearchReuest
from tiny_brave.datamodels.requests.news import NewsSearchRequest
from tiny_brave.datamodels.requests.videos import VideoSearchRequest
from tiny_brave.datamodels.requests.web import WebSearchRequest


async def brave_news_search(data: NewsSearchRequest) -> dict[str, Any]:
    """Perform a news search using the Brave Search API."""

    result = await TinyBraveClient().news(data)
    return result.model_dump()


async def brave_web_search(data: WebSearchRequest) -> dict[str, Any]:
    """Perform a web search using the Brave Search API."""

    result = await TinyBraveClient().web(data)
    return result.model_dump()


async def brave_images_search(data: ImagesSearchReuest) -> dict[str, Any]:
    """Perform an image search using the Brave Search API."""

    result = await TinyBraveClient().images(data)
    return result.model_dump()


async def brave_videos_search(data: VideoSearchRequest) -> dict[str, Any]:
    """Perform a video search using the Brave Search API."""

    result = await TinyBraveClient().videos(data)
    return result.model_dump()


if __name__ == '__main__':

    async def main():
        news = await brave_news_search(
            NewsSearchRequest(
                q='Brave Search API',
            )
        )
        print('News Search result: %s', news)

        web = await brave_web_search(
            WebSearchRequest(
                q='Brave Search API',
            )
        )
        print('Web Search result: %s', web)

        images = await brave_images_search(
            ImagesSearchReuest(
                q='Brave Search API',
            )
        )
        print('Image Search result: %s', images)

        videos = await brave_videos_search(
            VideoSearchRequest(
                q='Brave Search API',
            )
        )
        print('Video Search result: %s', videos)

    import asyncio

    asyncio.run(main())



from typing import List
from typing import Literal
from typing import Optional

from pydantic import Field

from tinygent.core.types.base import TinyModel


class Thumbnail(TinyModel):
    """Aggregated details representing the news thumbnail."""

    src: str = Field(
        ...,
        description='The served URL of the thumbnail associated with the news article.',
    )
    original: Optional[str] = Field(
        None,
        description='The original URL of the thumbnail associated with the news article.',
    )


class MetaUrl(TinyModel):
    """Aggregated information about a URL."""

    scheme: Optional[str] = Field(
        None, description='The protocol scheme extracted from the URL.'
    )
    netloc: Optional[str] = Field(
        None, description='The network location part extracted from the URL.'
    )
    hostname: Optional[str] = Field(
        None, description='The lowercased domain name extracted from the URL.'
    )
    favicon: Optional[str] = Field(None, description='The favicon used for the URL.')
    path: Optional[str] = Field(
        None, description='The hierarchical path of the URL useful as a display string.'
    )


class NewsResult(TinyModel):
    """A model representing a news result for the requested query."""

    type: str = Field(
        ...,
        description='The type of news search API result. The value is always news_result.',
    )
    url: str = Field(..., description='The source URL of the news article.')
    title: str = Field(..., description='The title of the news article.')
    description: Optional[str] = Field(
        None, description='The description for the news article.'
    )
    age: Optional[str] = Field(
        None, description='A human readable representation of the page age.'
    )
    page_age: Optional[str] = Field(
        None, description='The page age found from the source web page.'
    )
    page_fetched: Optional[str] = Field(
        None,
        description='The ISO date time when the page was last fetched. Format: YYYY-MM-DDTHH:MM:SSZ.',
    )
    breaking: Optional[bool] = Field(
        None, description='Whether the result includes breaking news.'
    )
    thumbnail: Optional[Thumbnail] = Field(
        None, description='The thumbnail for the news article.'
    )
    meta_url: Optional[MetaUrl] = Field(
        None,
        description='Aggregated information on the URL associated with the news search result.',
    )
    extra_snippets: Optional[List[str]] = Field(
        None,
        description='A list of extra alternate snippets for the news search result.',
    )


class Query(TinyModel):
    """A model representing information gathered around the requested query."""

    original: str = Field(..., description='The original query that was requested.')
    altered: Optional[str] = Field(
        None,
        description='The altered query by the spellchecker. '
        'This is the query that is used to search if any.',
    )
    cleaned: Optional[str] = Field(
        None,
        description='The cleaned normalized query by the spellchecker. '
        'This is the query that is used to search if any.',
    )
    spellcheck_off: Optional[bool] = Field(
        None, description='Whether the spellchecker is enabled or disabled.'
    )
    show_strict_warning: Optional[bool] = Field(
        None,
        description=(
            'True if lack of results is due to strict safesearch setting '
            '(adult content blocked).'
        ),
    )


class NewsSearchApiResponse(TinyModel):
    """Top level response model for successful News Search API requests."""

    type: Literal['news'] = Field('news', description='The type of search API result.')
    query: Query = Field(..., description='News search query string.')
    results: List[NewsResult] = Field(
        ..., description='The list of news results for the given query.'
    )



from tiny_brave.tools import brave_images_search
from tiny_brave.tools import brave_news_search
from tiny_brave.tools import brave_videos_search
from tiny_brave.tools import brave_web_search
from tinygent.tools.tool import register_tool


def _register_tools() -> None:
    register_tool(hidden=False)(brave_news_search)
    register_tool(hidden=False)(brave_web_search)
    register_tool(hidden=False)(brave_images_search)
    register_tool(hidden=False)(brave_videos_search)


_register_tools()



from pydantic import BaseModel
from pydantic import Field
from pydantic import ValidationError
from pydantic import create_model
from pydantic import field_validator
from pydantic import model_validator

from tiny_brave.constants import MAX_QUERY_LENGTH
from tiny_brave.constants import MAX_QUERY_TERMS
from tiny_brave.exceptions import TinyBraveClientError
from tinygent.core.types.base import TinyModel


class BaseSearchRequest(TinyModel):
    q: str = Field(
        ...,
        min_length=1,
        max_length=MAX_QUERY_LENGTH,
        description=(
            f'The search query string used to find relevant news articles. '
            f'Maximum length is {MAX_QUERY_LENGTH} characters.'
        ),
    )

    country: str | None = Field(
        default='US',
        description='2-character country code for where the results come from.',
    )

    spellcheck: bool = Field(
        default=False,
        description='Whether to enable spellchecking for the search query.',
    )

    search_lang: str | None = Field(
        default=None,
        description=(
            'The search language preference. The 2 or more character language code for '
            'which the search results are provided.'
        ),
    )

    @field_validator('q')
    def validate_query(cls, value: str) -> str:
        if len(value.strip().split()) > MAX_QUERY_TERMS:
            raise TinyBraveClientError(
                f'Query exceeds maximum term limit of {MAX_QUERY_TERMS}.'
            )
        return value

    @model_validator(mode='before')
    def sanitize(cls, values: dict) -> dict:
        for name, field in cls.model_fields.items():
            if name not in values:
                continue
            v = values[name]
            if v is None:
                continue

            Tmp = create_model(  # type: ignore
                f'_Tmp_{cls.__name__}_{name}',
                __base__=BaseModel,
                **{name: (field.annotation, field)},  # type: ignore
            )

            try:
                validated = Tmp(**{name: v})
                values[name] = getattr(validated, name)
            except ValidationError:
                values[name] = field.default

        return values



from typing import Literal
from typing import Optional

from pydantic import Field

from tinygent.core.types.base import TinyModel


class Query(TinyModel):
    original: str = Field(..., description='The original query that was requested.')
    altered: Optional[str] = Field(
        None, description='The altered query by the spellchecker.'
    )
    spellcheck_off: Optional[bool] = Field(
        None, description='Whether the spellchecker is enabled or disabled.'
    )
    show_strict_warning: Optional[bool] = Field(
        None, description='True if results blocked by strict safesearch setting.'
    )


class Thumbnail(TinyModel):
    src: Optional[str] = Field(None, description='The served URL of the image.')
    width: Optional[int] = Field(None, description='The width of the image.')
    height: Optional[int] = Field(None, description='The height of the image.')


class Properties(TinyModel):
    url: Optional[str] = Field(None, description='The image URL.')
    placeholder: Optional[str] = Field(None, description='The placeholder image URL.')
    width: Optional[int] = Field(None, description='The width of the image.')
    height: Optional[int] = Field(None, description='The height of the image.')


class MetaUrl(TinyModel):
    scheme: Optional[str] = Field(None, description='The protocol scheme from the URL.')
    netloc: Optional[str] = Field(
        None, description='The network location part from the URL.'
    )
    hostname: Optional[str] = Field(
        None, description='The lowercased domain name from the URL.'
    )
    favicon: Optional[str] = Field(None, description='The favicon used for the URL.')
    path: Optional[str] = Field(None, description='The hierarchical path of the URL.')


class ImageResult(TinyModel):
    type: Literal['image_result'] = Field(
        'image_result', description='The type of image search API result.'
    )
    title: Optional[str] = Field(None, description='The title of the image.')
    url: Optional[str] = Field(
        None, description='The original page URL where the image was found.'
    )
    source: Optional[str] = Field(
        None, description='The source domain where the image was found.'
    )
    page_fetched: Optional[str] = Field(
        None, description='ISO datetime when the page was last fetched.'
    )
    thumbnail: Optional[Thumbnail] = Field(
        None, description='The thumbnail for the image.'
    )
    properties: Optional[Properties] = Field(None, description='Metadata for the image.')
    meta_url: Optional[MetaUrl] = Field(
        None, description='Aggregated information on the image URL.'
    )
    confidence: Optional[Literal['low', 'medium', 'high']] = Field(
        None, description='The confidence level for the image result.'
    )


class Extra(TinyModel):
    might_be_offensive: bool = Field(
        ..., description='Indicates if results might be offensive.'
    )


class ImageSearchApiResponse(TinyModel):
    type: Literal['images'] = Field(
        'images', description='The type of search API result.'
    )
    query: Query = Field(..., description='Image search query string.')
    results: list[ImageResult] = Field(
        ..., description='List of image results for the query.'
    )
    extra: Extra = Field(..., description='Additional information about the results.')



from typing import Any
from typing import Literal

from pydantic import Field

from tinygent.core.types.base import TinyModel


class Thumbnail(TinyModel):
    src: str = Field(
        ...,
        description='The served URL of the picture thumbnail.',
    )
    original: str | None = Field(
        default=None,
        description='The original image URL.',
    )


class MetaUrl(TinyModel):
    scheme: str = Field(
        ...,
        description='The protocol scheme extracted from the URL.',
    )
    netloc: str = Field(
        ...,
        description='The network location part extracted from the URL.',
    )
    hostname: str | None = Field(
        default=None,
        description='The lowercased domain name extracted from the URL.',
    )
    favicon: str = Field(
        ...,
        description='The favicon URL associated with the domain.',
    )
    path: str = Field(
        ...,
        description='The hierarchical path of the URL.',
    )


class Query(TinyModel):
    original: str = Field(
        ...,
        description='The original query that was requested.',
    )
    altered: str | None = Field(
        default=None,
        description='The altered query for which the search was performed.',
    )
    safesearch: bool | None = Field(
        default=None,
        description='Whether safesearch was enabled.',
    )
    is_trending: bool | None = Field(
        default=None,
        description='Whether the query is trending.',
    )
    is_news_breaking: bool | None = Field(
        default=None,
        description='Whether the query has breaking news articles relevant to it.',
    )
    language: dict[str, Any] | None = Field(
        default=None,
        description='Language information gathered from the query.',
    )
    country: str | None = Field(
        default=None,
        description='The country that was used for the query.',
    )


class DiscussionResult(TinyModel):
    type: str = Field(
        default='discussion',
        description='The discussion result type identifier. Always "discussion".',
    )
    data: dict[str, Any] | None = Field(
        default=None,
        description='Enriched aggregated data for the relevant forum post.',
    )


class Discussions(TinyModel):
    type: str = Field(
        default='search',
        description='The type identifying a discussion cluster. Always "search".',
    )
    results: list[DiscussionResult] = Field(
        default_factory=list,
        description='A list of discussion results relevant to the query.',
    )
    mutated_by_goggles: bool | None = Field(
        default=None,
        description='Whether the discussion results were modified by a Goggle.',
    )


class QA(TinyModel):
    question: str = Field(
        ...,
        description='The question being asked.',
    )
    answer: str = Field(
        ...,
        description='The answer to the question.',
    )
    title: str = Field(
        ...,
        description='The title of the post containing the Q&A.',
    )
    url: str = Field(
        ...,
        description='The URL pointing to the post.',
    )
    meta_url: MetaUrl | None = Field(
        default=None,
        description='Aggregated information about the URL.',
    )


class FAQ(TinyModel):
    type: str = Field(
        default='faq',
        description='The FAQ result type identifier. Always "faq".',
    )
    results: list[QA] = Field(
        default_factory=list,
        description='A list of aggregated question-answer results relevant to the query.',
    )


class WebResult(TinyModel):
    type: str = Field(
        default='search_result',
        description='The type identifying a web search result. Always "search_result".',
    )
    title: str = Field(
        ...,
        description='The title of the web page.',
    )
    url: str = Field(
        ...,
        description='The URL of the web page.',
    )
    description: str | None = Field(
        default=None,
        description='A description or snippet of the web page.',
    )
    language: str | None = Field(
        default=None,
        description='The main language on the web page.',
    )
    family_friendly: bool | None = Field(
        default=None,
        description='Whether the web page is family friendly.',
    )
    meta_url: MetaUrl | None = Field(
        default=None,
        description='Aggregated metadata about the URL.',
    )
    thumbnail: Thumbnail | None = Field(
        default=None,
        description='Thumbnail image for the result.',
    )
    age: str | None = Field(
        default=None,
        description='A string representing the age of the web page.',
    )
    extra_snippets: list[str] | None = Field(
        default=None,
        description='A list of extra alternate snippets for the result.',
    )


class WebResults(TinyModel):
    type: str = Field(
        default='search',
        description='The type identifying web search results. Always "search".',
    )
    results: list[WebResult] = Field(
        default_factory=list,
        description='A list of web search results relevant to the query.',
    )
    family_friendly: bool | None = Field(
        default=None,
        description='Whether the search results are family friendly.',
    )


class NewsResult(TinyModel):
    meta_url: MetaUrl | None = Field(
        default=None,
        description='Aggregated metadata about the news article URL.',
    )
    source: str | None = Field(
        default=None,
        description='The source of the news.',
    )
    breaking: bool = Field(
        ...,
        description='Whether the news result is breaking news.',
    )
    is_live: bool = Field(
        ...,
        description='Whether the news result is live.',
    )
    thumbnail: Thumbnail | None = Field(
        default=None,
        description='Thumbnail image for the news result.',
    )
    age: str | None = Field(
        default=None,
        description='A string representing the age of the news article.',
    )
    extra_snippets: list[str] | None = Field(
        default=None,
        description='Extra alternate snippets for the news article.',
    )


class NewsResults(TinyModel):
    type: str = Field(
        default='news',
        description='The type identifying news results. Always "news".',
    )
    results: list[NewsResult] = Field(
        default_factory=list,
        description='A list of news results relevant to the query.',
    )
    mutated_by_goggles: bool | None = Field(
        default=None,
        description='Whether the news results were modified by a Goggle.',
    )


class VideoResult(TinyModel):
    type: str = Field(
        default='video_result',
        description='The type identifying a video result. Always "video_result".',
    )
    video: dict[str, Any] = Field(
        ...,
        description='Metadata for the video.',
    )
    meta_url: MetaUrl | None = Field(
        default=None,
        description='Aggregated metadata about the video URL.',
    )
    thumbnail: Thumbnail | None = Field(
        default=None,
        description='Thumbnail image for the video result.',
    )
    age: str | None = Field(
        default=None,
        description='A string representing the age of the video.',
    )


class VideoResults(TinyModel):
    type: str = Field(
        default='videos',
        description='The type identifying video results. Always "videos".',
    )
    results: list[VideoResult] = Field(
        default_factory=list,
        description='A list of video results relevant to the query.',
    )
    mutated_by_goggles: bool | None = Field(
        default=None,
        description='Whether the video results were modified by a Goggle.',
    )


class WebSearchApiResponse(TinyModel):
    type: Literal['search'] = Field(
        'search',
        description='The type of web search API result',
    )
    query: Query | None = Field(
        default=None,
        description='Search query and its modifications.',
    )
    web: WebResults | None = Field(
        default=None,
        description='Web search results relevant to the query.',
    )
    news: NewsResults | None = Field(
        default=None,
        description='News results relevant to the query.',
    )
    videos: VideoResults | None = Field(
        default=None,
        description='Videos relevant to the query.',
    )
    discussions: Discussions | None = Field(
        default=None,
        description='Discussions aggregated from forum posts relevant to the query.',
    )
    faq: FAQ | None = Field(
        default=None,
        description='Frequently asked questions relevant to the query.',
    )
    infobox: dict[str, Any] | None = Field(
        default=None,
        description='Aggregated information on an entity shown as an infobox.',
    )
    locations: dict[str, Any] | None = Field(
        default=None,
        description='Places of interest relevant to location-sensitive queries.',
    )
    summarizer: dict[str, Any] | None = Field(
        default=None,
        description='Summary key to get summary results for the query.',
    )
    rich: dict[str, Any] | None = Field(
        default=None,
        description='Callback information for rich results.',
    )
    mixed: dict[str, Any] | None = Field(
        default=None,
        description='Preferred ranked order of search results.',
    )



from .client import TinyBraveClient
from .datamodels.requests.images import ImagesSearchReuest
from .datamodels.requests.news import NewsSearchRequest
from .datamodels.requests.videos import VideoSearchRequest
from .datamodels.requests.web import WebSearchRequest
from .datamodels.responses.images import ImageSearchApiResponse
from .datamodels.responses.news import NewsSearchApiResponse
from .datamodels.responses.videos import VideoSearchApiResponse
from .datamodels.responses.web import WebSearchApiResponse
from .exceptions import TinyBraveAPIError
from .exceptions import TinyBraveClientError
from .exceptions import TinyBraveError
from .tools import brave_news_search
from .tools import brave_web_search

__all__ = [
    'TinyBraveClient',
    'TinyBraveError',
    'TinyBraveClientError',
    'TinyBraveAPIError',
    'NewsSearchRequest',
    'ImagesSearchReuest',
    'WebSearchRequest',
    'VideoSearchRequest',
    'NewsSearchApiResponse',
    'ImageSearchApiResponse',
    'WebSearchApiResponse',
    'VideoSearchApiResponse',
    'brave_news_search',
    'brave_web_search',
]



class TinyBraveError(Exception):
    """Base class for all TinyBrave exceptions."""


class TinyBraveClientError(TinyBraveError):
    """Exception raised for client-side errors."""


class TinyBraveAPIError(TinyBraveError):
    """Exception raised for API-side errors."""
